@@ -15,15 +15,15 @@
 // specific language governing permissions and limitations
 // under the License.
 
-use crate::aws::STRICT_ENCODE_SET;
+use crate::aws::{STORE, STRICT_ENCODE_SET, STRICT_PATH_ENCODE_SET};
 use crate::client::retry::RetryExt;
 use crate::client::token::{TemporaryToken, TokenCache};
+use crate::client::TokenProvider;
 use crate::util::hmac_sha256;
 use crate::{Result, RetryConfig};
+use async_trait::async_trait;
 use bytes::Buf;
 use chrono::{DateTime, Utc};
-use futures::future::BoxFuture;
-use futures::TryFutureExt;
 use percent_encoding::utf8_percent_encode;
 use reqwest::header::{HeaderMap, HeaderValue};
 use reqwest::{Client, Method, Request, RequestBuilder, StatusCode};
@@ -39,12 +39,17 @@ type StdError = Box<dyn std::error::Error + Send + Sync>;
 /// SHA256 hash of empty string
 static EMPTY_SHA256_HASH: &str =
     "e3b0c44298fc1c149afbf4c8996fb92427ae41e4649b934ca495991b7852b855";
-static UNSIGNED_PAYLOAD_LITERAL: &str = "UNSIGNED-PAYLOAD";
+static UNSIGNED_PAYLOAD: &str = "UNSIGNED-PAYLOAD";
+static STREAMING_PAYLOAD: &str = "STREAMING-AWS4-HMAC-SHA256-PAYLOAD";
 
-#[derive(Debug)]
+/// A set of AWS security credentials
+#[derive(Debug, Eq, PartialEq)]
 pub struct AwsCredential {
+    /// AWS_ACCESS_KEY_ID
     pub key_id: String,
+    /// AWS_SECRET_ACCESS_KEY
     pub secret_key: String,
+    /// AWS_SESSION_TOKEN
     pub token: Option<String>,
 }
 
@@ -68,8 +73,12 @@ impl AwsCredential {
     }
 }
 
-struct RequestSigner<'a> {
-    date: DateTime<Utc>,
+/// Authorize a [`Request`] with an [`AwsCredential`] using [AWS SigV4]
+///
+/// [AWS SigV4]: https://docs.aws.amazon.com/general/latest/gr/sigv4-calculate-signature.html
+#[derive(Debug)]
+pub struct AwsAuthorizer<'a> {
+    date: Option<DateTime<Utc>>,
     credential: &'a AwsCredential,
     service: &'a str,
     region: &'a str,
@@ -81,41 +90,78 @@ const HASH_HEADER: &str = "x-amz-content-sha256";
 const TOKEN_HEADER: &str = "x-amz-security-token";
 const AUTH_HEADER: &str = "authorization";
 
-const ALL_HEADERS: &[&str; 4] = &[DATE_HEADER, HASH_HEADER, TOKEN_HEADER, AUTH_HEADER];
+impl<'a> AwsAuthorizer<'a> {
+    /// Create a new [`AwsAuthorizer`]
+    pub fn new(credential: &'a AwsCredential, service: &'a str, region: &'a str) -> Self {
+        Self {
+            credential,
+            service,
+            region,
+            date: None,
+            sign_payload: true,
+        }
+    }
 
-impl<'a> RequestSigner<'a> {
-    fn sign(&self, request: &mut Request, pre_calculated_digest: Option<Vec<u8>>) {
+    /// Controls whether this [`AwsAuthorizer`] will attempt to sign the request payload,
+    /// the default is `true`
+    pub fn with_sign_payload(mut self, signed: bool) -> Self {
+        self.sign_payload = signed;
+        self
+    }
+
+    /// Authorize `request` with an optional pre-calculated SHA256 digest by attaching
+    /// the relevant [AWS SigV4] headers
+    ///
+    /// # Payload Signature
+    ///
+    /// AWS SigV4 requests must contain the `x-amz-content-sha256` header, it is set as follows:
+    ///
+    /// * If not configured to sign payloads, it is set to `UNSIGNED-PAYLOAD`
+    /// * If a `pre_calculated_digest` is provided, it is set to the hex encoding of it
+    /// * If it is a streaming request, it is set to `STREAMING-AWS4-HMAC-SHA256-PAYLOAD`
+    /// * Otherwise it is set to the hex encoded SHA256 of the request body
+    ///
+    /// [AWS SigV4]: https://docs.aws.amazon.com/IAM/latest/UserGuide/create-signed-request.html
+    pub fn authorize(&self, request: &mut Request, pre_calculated_digest: Option<&[u8]>) {
         if let Some(ref token) = self.credential.token {
             let token_val = HeaderValue::from_str(token).unwrap();
             request.headers_mut().insert(TOKEN_HEADER, token_val);
         }
 
-        let host_val = HeaderValue::from_str(
-            &request.url()[url::Position::BeforeHost..url::Position::AfterPort],
-        )
-        .unwrap();
+        let host = &request.url()[url::Position::BeforeHost..url::Position::AfterPort];
+        let host_val = HeaderValue::from_str(host).unwrap();
         request.headers_mut().insert("host", host_val);
 
-        let date_str = self.date.format("%Y%m%dT%H%M%SZ").to_string();
+        let date = self.date.unwrap_or_else(Utc::now);
+        let date_str = date.format("%Y%m%dT%H%M%SZ").to_string();
         let date_val = HeaderValue::from_str(&date_str).unwrap();
         request.headers_mut().insert(DATE_HEADER, date_val);
 
-        let digest = if self.sign_payload {
-            if let Some(digest) = pre_calculated_digest {
-                hex_encode(&digest)
-            } else {
-                match request.body() {
+        let digest = match self.sign_payload {
+            false => UNSIGNED_PAYLOAD.to_string(),
+            true => match pre_calculated_digest {
+                Some(digest) => hex_encode(digest),
+                None => match request.body() {
                     None => EMPTY_SHA256_HASH.to_string(),
-                    Some(body) => hex_digest(body.as_bytes().unwrap()),
-                }
-            }
-        } else {
-            UNSIGNED_PAYLOAD_LITERAL.to_string()
+                    Some(body) => match body.as_bytes() {
+                        Some(bytes) => hex_digest(bytes),
+                        None => STREAMING_PAYLOAD.to_string(),
+                    },
+                },
+            },
         };
 
         let header_digest = HeaderValue::from_str(&digest).unwrap();
         request.headers_mut().insert(HASH_HEADER, header_digest);
 
+        // Each path segment must be URI-encoded twice (except for Amazon S3 which only gets URI-encoded once).
+        // see https://docs.aws.amazon.com/general/latest/gr/sigv4-create-canonical-request.html
+        let canonical_uri = match self.service {
+            "s3" => request.url().path().to_string(),
+            _ => utf8_percent_encode(request.url().path(), &STRICT_PATH_ENCODE_SET)
+                .to_string(),
+        };
+
         let (signed_headers, canonical_headers) = canonicalize_headers(request.headers());
         let canonical_query = canonicalize_query(request.url());
 
@@ -123,7 +169,7 @@ impl<'a> RequestSigner<'a> {
         let canonical_request = format!(
             "{}\n{}\n{}\n{}\n{}\n{}",
             request.method().as_str(),
-            request.url().path(), // S3 doesn't percent encode this like other services
+            canonical_uri,
             canonical_query,
             canonical_headers,
             signed_headers,
@@ -133,14 +179,14 @@ impl<'a> RequestSigner<'a> {
         let hashed_canonical_request = hex_digest(canonical_request.as_bytes());
         let scope = format!(
             "{}/{}/{}/aws4_request",
-            self.date.format("%Y%m%d"),
+            date.format("%Y%m%d"),
             self.region,
             self.service
         );
 
         let string_to_sign = format!(
             "AWS4-HMAC-SHA256\n{}\n{}\n{}",
-            self.date.format("%Y%m%dT%H%M%SZ"),
+            date.format("%Y%m%dT%H%M%SZ"),
             scope,
             hashed_canonical_request
         );
@@ -148,7 +194,7 @@ impl<'a> RequestSigner<'a> {
         // sign the string
         let signature =
             self.credential
-                .sign(&string_to_sign, self.date, self.region, self.service);
+                .sign(&string_to_sign, date, self.region, self.service);
 
         // build the actual auth header
         let authorisation = format!(
@@ -169,44 +215,27 @@ pub trait CredentialExt {
         region: &str,
         service: &str,
         sign_payload: bool,
-        payload_sha256: Option<Vec<u8>>,
+        payload_sha256: Option<&[u8]>,
     ) -> Self;
 }
 
 impl CredentialExt for RequestBuilder {
     fn with_aws_sigv4(
-        mut self,
+        self,
         credential: &AwsCredential,
         region: &str,
         service: &str,
         sign_payload: bool,
-        payload_sha256: Option<Vec<u8>>,
+        payload_sha256: Option<&[u8]>,
     ) -> Self {
-        // Hack around lack of access to underlying request
-        // https://github.com/seanmonstar/reqwest/issues/1212
-        let mut request = self
-            .try_clone()
-            .expect("not stream")
-            .build()
-            .expect("request valid");
+        let (client, request) = self.build_split();
+        let mut request = request.expect("request valid");
 
-        let date = Utc::now();
-        let signer = RequestSigner {
-            date,
-            credential,
-            service,
-            region,
-            sign_payload,
-        };
+        AwsAuthorizer::new(credential, service, region)
+            .with_sign_payload(sign_payload)
+            .authorize(&mut request, payload_sha256);
 
-        signer.sign(&mut request, payload_sha256);
-
-        for header in ALL_HEADERS {
-            if let Some(val) = request.headers_mut().remove(*header) {
-                self = self.header(*header, val)
-            }
-        }
-        self
+        Self::from_parts(client, request)
     }
 }
 
@@ -304,49 +333,31 @@ fn canonicalize_headers(header_map: &HeaderMap) -> (String, String) {
     (signed_headers, canonical_headers)
 }
 
-/// Provides credentials for use when signing requests
-pub trait CredentialProvider: std::fmt::Debug + Send + Sync {
-    fn get_credential(&self) -> BoxFuture<'_, Result<Arc<AwsCredential>>>;
-}
-
-/// A static set of credentials
-#[derive(Debug)]
-pub struct StaticCredentialProvider {
-    pub credential: Arc<AwsCredential>,
-}
-
-impl CredentialProvider for StaticCredentialProvider {
-    fn get_credential(&self) -> BoxFuture<'_, Result<Arc<AwsCredential>>> {
-        Box::pin(futures::future::ready(Ok(Arc::clone(&self.credential))))
-    }
-}
-
 /// Credentials sourced from the instance metadata service
 ///
 /// <https://docs.aws.amazon.com/AWSEC2/latest/UserGuide/configuring-instance-metadata-service.html>
 #[derive(Debug)]
 pub struct InstanceCredentialProvider {
     pub cache: TokenCache<Arc<AwsCredential>>,
-    pub client: Client,
-    pub retry_config: RetryConfig,
     pub imdsv1_fallback: bool,
     pub metadata_endpoint: String,
 }
 
-impl CredentialProvider for InstanceCredentialProvider {
-    fn get_credential(&self) -> BoxFuture<'_, Result<Arc<AwsCredential>>> {
-        Box::pin(self.cache.get_or_insert_with(|| {
-            instance_creds(
-                &self.client,
-                &self.retry_config,
-                &self.metadata_endpoint,
-                self.imdsv1_fallback,
-            )
+#[async_trait]
+impl TokenProvider for InstanceCredentialProvider {
+    type Credential = AwsCredential;
+
+    async fn fetch_token(
+        &self,
+        client: &Client,
+        retry: &RetryConfig,
+    ) -> Result<TemporaryToken<Arc<AwsCredential>>> {
+        instance_creds(client, retry, &self.metadata_endpoint, self.imdsv1_fallback)
+            .await
             .map_err(|source| crate::Error::Generic {
-                store: "S3",
+                store: STORE,
                 source,
             })
-        }))
     }
 }
 
@@ -355,31 +366,34 @@ impl CredentialProvider for InstanceCredentialProvider {
 /// <https://docs.aws.amazon.com/eks/latest/userguide/iam-roles-for-service-accounts-technical-overview.html>
 #[derive(Debug)]
 pub struct WebIdentityProvider {
-    pub cache: TokenCache<Arc<AwsCredential>>,
     pub token_path: String,
     pub role_arn: String,
     pub session_name: String,
     pub endpoint: String,
-    pub client: Client,
-    pub retry_config: RetryConfig,
 }
 
-impl CredentialProvider for WebIdentityProvider {
-    fn get_credential(&self) -> BoxFuture<'_, Result<Arc<AwsCredential>>> {
-        Box::pin(self.cache.get_or_insert_with(|| {
-            web_identity(
-                &self.client,
-                &self.retry_config,
-                &self.token_path,
-                &self.role_arn,
-                &self.session_name,
-                &self.endpoint,
-            )
-            .map_err(|source| crate::Error::Generic {
-                store: "S3",
-                source,
-            })
-        }))
+#[async_trait]
+impl TokenProvider for WebIdentityProvider {
+    type Credential = AwsCredential;
+
+    async fn fetch_token(
+        &self,
+        client: &Client,
+        retry: &RetryConfig,
+    ) -> Result<TemporaryToken<Arc<AwsCredential>>> {
+        web_identity(
+            client,
+            retry,
+            &self.token_path,
+            &self.role_arn,
+            &self.session_name,
+            &self.endpoint,
+        )
+        .await
+        .map_err(|source| crate::Error::Generic {
+            store: STORE,
+            source,
+        })
     }
 }
 
@@ -528,68 +542,6 @@ async fn web_identity(
     })
 }
 
-#[cfg(feature = "aws_profile")]
-mod profile {
-    use super::*;
-    use aws_config::profile::ProfileFileCredentialsProvider;
-    use aws_config::provider_config::ProviderConfig;
-    use aws_credential_types::provider::ProvideCredentials;
-    use aws_types::region::Region;
-    use std::time::SystemTime;
-
-    #[derive(Debug)]
-    pub struct ProfileProvider {
-        cache: TokenCache<Arc<AwsCredential>>,
-        credentials: ProfileFileCredentialsProvider,
-    }
-
-    impl ProfileProvider {
-        pub fn new(name: String, region: String) -> Self {
-            let config = ProviderConfig::default().with_region(Some(Region::new(region)));
-
-            Self {
-                cache: Default::default(),
-                credentials: ProfileFileCredentialsProvider::builder()
-                    .configure(&config)
-                    .profile_name(name)
-                    .build(),
-            }
-        }
-    }
-
-    impl CredentialProvider for ProfileProvider {
-        fn get_credential(&self) -> BoxFuture<'_, Result<Arc<AwsCredential>>> {
-            Box::pin(self.cache.get_or_insert_with(move || async move {
-                let c =
-                    self.credentials
-                        .provide_credentials()
-                        .await
-                        .map_err(|source| crate::Error::Generic {
-                            store: "S3",
-                            source: Box::new(source),
-                        })?;
-                let t_now = SystemTime::now();
-                let expiry = c
-                    .expiry()
-                    .and_then(|e| e.duration_since(t_now).ok())
-                    .map(|ttl| Instant::now() + ttl);
-
-                Ok(TemporaryToken {
-                    token: Arc::new(AwsCredential {
-                        key_id: c.access_key_id().to_string(),
-                        secret_key: c.secret_access_key().to_string(),
-                        token: c.session_token().map(ToString::to_string),
-                    }),
-                    expiry,
-                })
-            }))
-        }
-    }
-}
-
-#[cfg(feature = "aws_profile")]
-pub use profile::ProfileProvider;
-
 #[cfg(test)]
 mod tests {
     use super::*;
@@ -625,15 +577,15 @@ mod tests {
             .build()
             .unwrap();
 
-        let signer = RequestSigner {
-            date,
+        let signer = AwsAuthorizer {
+            date: Some(date),
             credential: &credential,
             service: "ec2",
             region: "us-east-1",
             sign_payload: true,
         };
 
-        signer.sign(&mut request, None);
+        signer.authorize(&mut request, None);
         assert_eq!(request.headers().get(AUTH_HEADER).unwrap(), "AWS4-HMAC-SHA256 Credential=AKIAIOSFODNN7EXAMPLE/20220806/us-east-1/ec2/aws4_request, SignedHeaders=host;x-amz-content-sha256;x-amz-date, Signature=a3c787a7ed37f7fdfbfd2d7056a3d7c9d85e6d52a2bfbec73793c0be6e7862d4")
     }
 
@@ -663,15 +615,15 @@ mod tests {
             .build()
             .unwrap();
 
-        let signer = RequestSigner {
-            date,
+        let authorizer = AwsAuthorizer {
+            date: Some(date),
             credential: &credential,
             service: "ec2",
             region: "us-east-1",
             sign_payload: false,
         };
 
-        signer.sign(&mut request, None);
+        authorizer.authorize(&mut request, None);
         assert_eq!(request.headers().get(AUTH_HEADER).unwrap(), "AWS4-HMAC-SHA256 Credential=AKIAIOSFODNN7EXAMPLE/20220806/us-east-1/ec2/aws4_request, SignedHeaders=host;x-amz-content-sha256;x-amz-date, Signature=653c3d8ea261fd826207df58bc2bb69fbb5003e9eb3c0ef06e4a51f2a81d8699")
     }
 
@@ -700,15 +652,15 @@ mod tests {
             .build()
             .unwrap();
 
-        let signer = RequestSigner {
-            date,
+        let authorizer = AwsAuthorizer {
+            date: Some(date),
             credential: &credential,
             service: "s3",
             region: "us-east-1",
             sign_payload: true,
         };
 
-        signer.sign(&mut request, None);
+        authorizer.authorize(&mut request, None);
         assert_eq!(request.headers().get(AUTH_HEADER).unwrap(), "AWS4-HMAC-SHA256 Credential=H20ABqCkLZID4rLe/20220809/us-east-1/s3/aws4_request, SignedHeaders=host;x-amz-content-sha256;x-amz-date, Signature=9ebf2f92872066c99ac94e573b4e1b80f4dbb8a32b1e8e23178318746e7d1b4d")
     }
 
