@@ -16,8 +16,8 @@
 // under the License.
 
 //! An in-memory object store implementation
-use crate::MultipartId;
 use crate::{path::Path, GetResult, ListResult, ObjectMeta, ObjectStore, Result};
+use crate::{GetOptions, MultipartId};
 use async_trait::async_trait;
 use bytes::Bytes;
 use chrono::{DateTime, Utc};
@@ -117,12 +117,28 @@ impl ObjectStore for InMemory {
         Ok(())
     }
 
-    async fn get(&self, location: &Path) -> Result<GetResult> {
-        let data = self.entry(location).await?;
+    async fn append(
+        &self,
+        location: &Path,
+    ) -> Result<Box<dyn AsyncWrite + Unpin + Send>> {
+        Ok(Box::new(InMemoryAppend {
+            location: location.clone(),
+            data: Vec::<u8>::new(),
+            storage: StorageType::clone(&self.storage),
+        }))
+    }
 
-        Ok(GetResult::Stream(
-            futures::stream::once(async move { Ok(data.0) }).boxed(),
-        ))
+    async fn get_opts(&self, location: &Path, options: GetOptions) -> Result<GetResult> {
+        if options.if_match.is_some() || options.if_none_match.is_some() {
+            return Err(super::Error::NotSupported {
+                source: "ETags not supported by InMemory".to_string().into(),
+            });
+        }
+        let (data, last_modified) = self.entry(location).await?;
+        options.check_modified(location, last_modified)?;
+
+        let stream = futures::stream::once(futures::future::ready(Ok(data)));
+        Ok(GetResult::Stream(stream.boxed()))
     }
 
     async fn get_range(&self, location: &Path, range: Range<usize>) -> Result<Bytes> {
@@ -151,10 +167,12 @@ impl ObjectStore for InMemory {
 
     async fn head(&self, location: &Path) -> Result<ObjectMeta> {
         let entry = self.entry(location).await?;
+
         Ok(ObjectMeta {
             location: location.clone(),
             last_modified: entry.1,
             size: entry.0.len(),
+            e_tag: None,
         })
     }
 
@@ -185,6 +203,7 @@ impl ObjectStore for InMemory {
                     location: key.clone(),
                     last_modified: value.1,
                     size: value.0.len(),
+                    e_tag: None,
                 })
             })
             .collect();
@@ -228,6 +247,7 @@ impl ObjectStore for InMemory {
                     location: k.clone(),
                     last_modified: v.1,
                     size: v.0.len(),
+                    e_tag: None,
                 };
                 objects.push(object);
             }
@@ -325,23 +345,65 @@ impl AsyncWrite for InMemoryUpload {
     }
 }
 
+struct InMemoryAppend {
+    location: Path,
+    data: Vec<u8>,
+    storage: StorageType,
+}
+
+impl AsyncWrite for InMemoryAppend {
+    fn poll_write(
+        mut self: Pin<&mut Self>,
+        _cx: &mut std::task::Context<'_>,
+        buf: &[u8],
+    ) -> std::task::Poll<Result<usize, io::Error>> {
+        self.data.extend_from_slice(buf);
+        Poll::Ready(Ok(buf.len()))
+    }
+
+    fn poll_flush(
+        mut self: Pin<&mut Self>,
+        _cx: &mut std::task::Context<'_>,
+    ) -> std::task::Poll<Result<(), io::Error>> {
+        let storage = StorageType::clone(&self.storage);
+
+        let mut writer = storage.write();
+
+        if let Some((bytes, _)) = writer.remove(&self.location) {
+            let buf = std::mem::take(&mut self.data);
+            let concat = Bytes::from_iter(bytes.into_iter().chain(buf.into_iter()));
+            writer.insert(self.location.clone(), (concat, Utc::now()));
+        } else {
+            writer.insert(
+                self.location.clone(),
+                (Bytes::from(std::mem::take(&mut self.data)), Utc::now()),
+            );
+        };
+        Poll::Ready(Ok(()))
+    }
+
+    fn poll_shutdown(
+        self: Pin<&mut Self>,
+        cx: &mut std::task::Context<'_>,
+    ) -> std::task::Poll<Result<(), io::Error>> {
+        self.poll_flush(cx)
+    }
+}
+
 #[cfg(test)]
 mod tests {
+    use tokio::io::AsyncWriteExt;
+
     use super::*;
 
-    use crate::{
-        tests::{
-            copy_if_not_exists, get_nonexistent_object, list_uses_directories_correctly,
-            list_with_delimiter, put_get_delete_list, rename_and_copy, stream_get,
-        },
-        Error as ObjectStoreError, ObjectStore,
-    };
+    use crate::tests::*;
 
     #[tokio::test]
     async fn in_memory_test() {
         let integration = InMemory::new();
 
         put_get_delete_list(&integration).await;
+        get_opts(&integration).await;
         list_uses_directories_correctly(&integration).await;
         list_with_delimiter(&integration).await;
         rename_and_copy(&integration).await;
@@ -381,7 +443,7 @@ mod tests {
         let err = get_nonexistent_object(&integration, Some(location))
             .await
             .unwrap_err();
-        if let ObjectStoreError::NotFound { path, source } = err {
+        if let crate::Error::NotFound { path, source } = err {
             let source_variant = source.downcast_ref::<Error>();
             assert!(
                 matches!(source_variant, Some(Error::NoDataInMemory { .. }),),
@@ -392,4 +454,50 @@ mod tests {
             panic!("unexpected error type: {err:?}");
         }
     }
+
+    #[tokio::test]
+    async fn test_append_new() {
+        let in_memory = InMemory::new();
+        let location = Path::from("some_file");
+        let data = Bytes::from("arbitrary data");
+        let expected_data = data.clone();
+
+        let mut writer = in_memory.append(&location).await.unwrap();
+        writer.write_all(&data).await.unwrap();
+        writer.flush().await.unwrap();
+
+        let read_data = in_memory
+            .get(&location)
+            .await
+            .unwrap()
+            .bytes()
+            .await
+            .unwrap();
+        assert_eq!(&*read_data, expected_data);
+    }
+
+    #[tokio::test]
+    async fn test_append_existing() {
+        let in_memory = InMemory::new();
+        let location = Path::from("some_file");
+        let data = Bytes::from("arbitrary");
+        let data_appended = Bytes::from(" data");
+        let expected_data = Bytes::from("arbitrary data");
+
+        let mut writer = in_memory.append(&location).await.unwrap();
+        writer.write_all(&data).await.unwrap();
+        writer.flush().await.unwrap();
+
+        writer.write_all(&data_appended).await.unwrap();
+        writer.flush().await.unwrap();
+
+        let read_data = in_memory
+            .get(&location)
+            .await
+            .unwrap()
+            .bytes()
+            .await
+            .unwrap();
+        assert_eq!(&*read_data, expected_data);
+    }
 }
